unit gratings;
{ simulation de rseaux gravs par calcul dynamique complet }
{ F. OTT : 29.09.01                                         }
{ cette unit s'interface avec le programme SimulReflec     }

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type
  TGratingsForm = class(TForm)
  private
    { Dclarations prives}
  public
    { Dclarations publiques}
  end;

var
  GratingsForm: TGratingsForm;

implementation

{$R *.DFM}

{ Diffraction sur un reseau grav dans une multicouche  }
{ INPUT: lambda, longueur d'onde incidente              }
{         d, periode du grating                         }
{         anginc, angle d'incidence sur le grating      }
{         N, ordre auquel on fait le calcul             }
{ OUTPUT: * le vecteur des intensits diffractes des diffrents modes                       }
{         * les intensits des diffrents modes dans les couches elles memes                 }
{           cela peut servir pour le calcul du hors spculaire (calculs du HS en DWBA (Sinha)) }
{ methode matricielle
{ basee sur le papier de L. Li, J. Opt. Soc. Am. A Vol 13 May 1996
{ Formulation and comparison of two recursive matric algorithms for modeling layered diffraction gratings }
{ les couches sont indicees du substrat vers le vide            }
{ l'interface 0 est l'interface substrat/premiere couche        }
{ l'interface "virtuelle" -1 est dans le substrat               }
{ d periode du reseau                                           }
{ toutes les longueurs sont exprimees en nanometres             }
{ N ordre a lequel on fait le calcul                            }
procedure DiffractionReseauMulticoucheMtout(lambda,thetain:double);
var i,nbrLayers:integer;
begin


end;

end.

{ =============== FIN de l'unit =============================== }

% le fichier layers.txt contient la description du grating a simuler
% en partant du substrat vers le vide
% colonne 1: indice b du milieu considere (en fm)
% colonne 2: densite en 1e28 atomes par m3
% colonne 3: epaisseur de la couche en nm
% colonne 4: largeur des lignes en nm
load g:\simul\gratings\layers.txt;
% definition of some constants
nbrLayers=size(layers,1);
kvide=2*pi/lambda;
theta = anginc*pi/180;	% angle d'incidence (!! defini par rapport a la plan !!)
alpha0=kvide*cos(theta);% composante suivant x du vecteur d'onde = alpha0
a0 = 0;           		% epaisseur du substrat (interface fictive?!) pas clair du pourquoii c'est necessaire...)
                        % a priori on peut prendre cette interface a zero (ne change rien dans l'algorithme

% definition of the layers
b=layers(:,1); % -i*layers(:,3);
rho=layers(:,2);
n=sqrt(1-lambda*lambda*1e-18*rho*1e28.*b*1e-15/pi);     % indices des couches du grating
q=kvide*n;  % vecteur d'onde dans chacune des couches
a=layers(:,3);
d1=layers(:,4);
% on utilise l'algorithme (1a)
% cad on fait tout de maniere aussi explicite que possible
% et on utilise les formules de recursion (15a)
% on essaie de limiter la quantite de data stockees
% vecteurs d'onde du developpement de Rayleigh
one=eye(2*N+1);
% alpha=zeros(2*N+1,1);  *** WHY is it remmed ***
beta=zeros(2*N+1);
W0 = zeros(2*(2*N+1));
W1 = zeros(2*(2*N+1));
% initialisation de la matrice S^0
Tuu=one;Tdd=one;
Rud=zeros(2*N+1);Rdu=zeros(2*N+1);
t11=zeros(2*N+1);t12=zeros(2*N+1);t21=zeros(2*N+1);t22=zeros(2*N+1); % utile ?
% init substrat
for n=1:2*N+1,
   alpha(n)=alpha0+(n-N-1)*2*pi/d;
   beta(n,n)=sqrt(q(1)^2-alpha(n)^2);  
   if (imag(beta(n,n))+real(beta(n,n)))<0, beta(n,n)=-beta(n,n); end
end
ValP0=zeros(2*N+1);
% substrat
W0(1:2*N+1,1:2*N+1)=one;
W0(1:2*N+1,2*N+2:2*(2*N+1))=one;
W0(2*N+2:2*(2*N+1),1:2*N+1)=beta;
W0(2*N+2:2*(2*N+1),2*N+2:2*(2*N+1))=-W0(2*N+2:2*(2*N+1),1:2*N+1);
% initialisation de la recursion
% S(-1) = [ 1 0 ]
%         [ 0 1 ]
% etape p=-1; initialisation

% boucle principale
for nc=1:nbrLayers-1,
   % calcul des modes propres
   v=matricev(d,d1(nc+1)/2,kvide,q(nc+1),N,alpha); % attention a bien mettre d1/2 !
   [VectP,ValP]=eig(v); % calcul des vecteurs propres et des valeurs propres de la matrice v
   % il faut prendre la racine de ces valeurs propres et en faire des vp "up"
   % toutes les valeurs propres sont positives (ou a partie imaginaire positive)
   for k=1:2*N+1
      ValP(k,k)=sqrt(-ValP(k,k));
      if (imag(ValP(k,k))+real(ValP(k,k)))<0, ValP(k,k)=-ValP(k,k); end
   end
   %   VectP,ValP
   % tri des valeurs propres (algorithme tres rustique)
   NotFinished=1;
   while NotFinished,
      NotFinished=0;
      for k=1:2*N
         if (real(ValP(k,k))==0) & (imag(ValP(k+1,k+1))==0)
            aux=ValP(k,k);ValP(k,k)=ValP(k+1,k+1);ValP(k+1,k+1)=aux;
            aux=VectP(:,k);VectP(:,k)=VectP(:,k+1);VectP(:,k+1)=aux;
            NotFinished=1;
	    	end;
         if (imag(ValP(k,k))==0) & (imag(ValP(k+1,k+1))==0) & (abs(ValP(k,k))<abs(ValP(k+1,k+1)))
            aux=ValP(k,k);ValP(k,k)=ValP(k+1,k+1);ValP(k+1,k+1)=aux;
            aux=VectP(:,k);VectP(:,k)=VectP(:,k+1);VectP(:,k+1)=aux;
            NotFinished=1;
         end;
         if (real(ValP(k,k))==0) & (real(ValP(k+1,k+1))==0) & (abs(ValP(k,k))>abs(ValP(k+1,k+1)))
            aux=ValP(k,k);ValP(k,k)=ValP(k+1,k+1);ValP(k+1,k+1)=aux;
            aux=VectP(:,k);VectP(:,k)=VectP(:,k+1);VectP(:,k+1)=aux;
            NotFinished=1;
         end;
      end;
   end;
   %VectP,ValP

   % remplissage des matrices W determinant les conditions aux limites
   W1(1:2*N+1,1:2*N+1)=VectP;
   W1(1:2*N+1,2*N+2:2*(2*N+1))=VectP;
   W1(2*N+2:2*(2*N+1),1:2*N+1)=VectP*ValP;
   W1(2*N+2:2*(2*N+1),2*N+2:2*(2*N+1))=-W1(2*N+2:2*(2*N+1),1:2*N+1);
   %W1
   % calcul de la matrice t
   t=inv(W1)*W0;  % verifier la qualite des matrices pour l'inversion !!!
   % matrice t
   t11=t(1:2*N+1,1:2*N+1);
   t12=t(1:2*N+1,2*N+2:2*(2*N+1));
   t21=t(2*N+2:2*(2*N+1),1:2*N+1);
   t22=t(2*N+2:2*(2*N+1),2*N+2:2*(2*N+1));
   % remplissage de la matrice s (formules 14a)
   tdd=inv(t22);
   rdu=-tdd*t21;
   tuu=t11+t12*rdu;
   rud=t12*tdd;
   % facteur de phase dependant de l'epaisseur
   for k=1:2*N+1,   % NB ces deux matrices de propagation ne contiennent pas d'exponentielle croissantes dans le cas de milieux non absorbants!!
      aux=exp(i*ValP0(k,k)*a(nc));
      tuu(k,k)=tuu(k,k)*aux;
      rdu(k,k)=rdu(k,k)*aux*aux;
      tdd(k,k)=tdd(k,k)*aux;
   end
   % recursion
   aux1=inv(one-Rud*rdu)*Tuu;
   aux2=inv(one-rdu*Rud)*tdd;
   Tuu = tuu*aux1;
   Rud = rud+tuu*Rud*aux2;
   Rdu = Rdu+Tdd*rdu*aux1;
   Tdd = Tdd*aux2;
   W0=W1;
   ValP0=ValP; % on conserve les valeurs propres parce qu'on en a besoin pour la prochaine matrice de propagation
   % stockage des matrices S intermdiaires pour le calcul des amplitudes intermdiaires
   STdd(nc)=Tdd;SRud(nc)=Rud; % valable uniquement pour N=0
end
% intensite des modes diffractes (ok pour des couches continues)
% vecteur incident
dd3=zeros(2*N+1,1);dd3(N+1)=1;
dd1=Tdd*dd3;
uu3=Rud*dd3;
% out=abs(uu3).^2;
% out=abs(d1).^2;
% calcul des amplitudes des modes (u) et (d) dans les diffrentes couches
%  partir des matrices S intermdiaires
for nc=1:nbrLayers-1
   STdd(nc);
   SRud(nc);
   % disp('indice ');disp(nc+1);
   dd(nc+1)=dd1/STdd(nc);
   uu(nc+1)=SRud(nc)*dd(nc+1);
end
%abs(u3)^2
%abs(d1)^2
out=abs(uu).^2;
abs(dd')
abs(uu')


end.
